searchNodes=[{"doc":"Implements a timers queue based on a list of maps. The queue can be encoded as JSON. The performance will be worse for large queues in regard to the previous gb_trees based implementation, although the difference is negligible for small queues (&lt;= 1000 entries). All map keys are shrinked to a single letter as this queue is intended to be encoded and published to HTTP clients over the wire, mutiple times. The queue keys are a map composed of the timestamp ( t ) of an event and an unique integer ( u ). No erlang timers or processes are used, as the queue is only a data structure. The advantage is that the queue can be persisted on storage and keeps working after restarting the runtime. The queue maintain its own list of unique integers to avoir relying on BEAM unique integers as they are reset on VM restarts. The main drawback of a functional queue is that the queue entries must be manually checked for expired timers.","ref":"TimeQueue.html","title":"TimeQueue","type":"module"},{"doc":"Deletes an event from the queue and returns the new queue. It accepts a time reference or a full event. When an event is given, its time reference will be used to find the event to delete, meaning the queue event will be deleted even if the value of the passed event was tampered. The function does not fail if the event cannot be found and simply returns the queue as-is.","ref":"TimeQueue.html#delete/2","title":"TimeQueue.delete/2","type":"function"},{"doc":"Deletes all entries from the queue whose values are equal to unwanted .","ref":"TimeQueue.html#delete_val/2","title":"TimeQueue.delete_val/2","type":"function"},{"doc":"Adds a new event to the queue with a TTL and the current system time as now/0 . See enqueue/4 .","ref":"TimeQueue.html#enqueue/3","title":"TimeQueue.enqueue/3","type":"function"},{"doc":"Adds a new event to the queue with a TTL relative to the given timestamp in milliseconds. Returns {:ok, tref, new_queue} where tref is a timer reference.","ref":"TimeQueue.html#enqueue/4","title":"TimeQueue.enqueue/4","type":"function"},{"doc":"Adds a new event to the queue with an absolute timestamp. Returns {:ok, tref, new_queue} where tref is a timer reference.","ref":"TimeQueue.html#enqueue_abs/3","title":"TimeQueue.enqueue_abs/3","type":"function"},{"doc":"Returns a new queue with entries for whom the given callback returned a truthy value. Use filter_val/2 to filter only using values.","ref":"TimeQueue.html#filter/2","title":"TimeQueue.filter/2","type":"function"},{"doc":"Returns a new queue with entries for whom the given callback returned a truthy value. Unlinke filter/2 , the callback is only passed the event value.","ref":"TimeQueue.html#filter_val/2","title":"TimeQueue.filter_val/2","type":"function"},{"doc":"Creates an empty time queue. iex&gt; tq = TimeQueue . new ( ) iex&gt; TimeQueue . peek_event ( tq ) :empty","ref":"TimeQueue.html#new/0","title":"TimeQueue.new/0","type":"function"},{"doc":"This function is used internally to determine the current time when using functions enqueue/3 , pop/1 , pop_event/1 and peek_event/1 . It is a simple alias to :erlang.system_time(:millisecond) . TimeQueue does not use monotonic time since it already manages its own unique identifiers for queue entries.","ref":"TimeQueue.html#now/0","title":"TimeQueue.now/0","type":"function"},{"doc":"Returns the next value of the queue or a delay in milliseconds before the next value. See peek/2 .","ref":"TimeQueue.html#peek/1","title":"TimeQueue.peek/1","type":"function"},{"doc":"Returns the next value of the queue, or a delay, according to the given current time in milliseconds. Just like pop/2 vs. pop_event/2 , peek wil only return {:ok, value} when a timeout is reached whereas peek_event will return {:ok, event} .","ref":"TimeQueue.html#peek/2","title":"TimeQueue.peek/2","type":"function"},{"doc":"Alias for peek_event/1 .","ref":"TimeQueue.html#peek_entry/1","title":"TimeQueue.peek_entry/1","type":"function"},{"doc":"Alias for peek_event/2 .","ref":"TimeQueue.html#peek_entry/2","title":"TimeQueue.peek_entry/2","type":"function"},{"doc":"Returns the next event of the queue or a delay in milliseconds before the next value. event values can be retrieved with TimeQueue.value/1 . See peek_event/2 .","ref":"TimeQueue.html#peek_event/1","title":"TimeQueue.peek_event/1","type":"function"},{"doc":"Returns the next event of the queue according to the given current time in milliseconds. Possible return values are: :empty {:ok, event} if the timestamp of the first event is &lt;= to the given current time. {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue . new ( ) |&gt; TimeQueue . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue . peek_event ( tq , _now = 20 ) iex&gt; { :ok , _ } = TimeQueue . peek_event ( tq , _now = 100 )","ref":"TimeQueue.html#peek_event/2","title":"TimeQueue.peek_event/2","type":"function"},{"doc":"Extracts the next event in the queue or returns a delay. See pop/2 .","ref":"TimeQueue.html#pop/1","title":"TimeQueue.pop/1","type":"function"},{"doc":"Extracts the next event in the queue according to the given current time in milliseconds. Much like pop_event/2 but the tuple returned when an event time is reached (returns with :ok ) success will only contain the value inserted in the queue. Possible return values are: :empty {:ok, value, new_queue} if the timestamp of the first event is &lt;= to the given current time. The event is deleted from new_queue . {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue . new ( ) |&gt; TimeQueue . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue . pop ( tq , _now = 20 ) iex&gt; { :ok , value , _ } = TimeQueue . pop ( tq , _now = 100 ) iex&gt; value :hello","ref":"TimeQueue.html#pop/2","title":"TimeQueue.pop/2","type":"function"},{"doc":"Alias for pop_event/1 .","ref":"TimeQueue.html#pop_entry/1","title":"TimeQueue.pop_entry/1","type":"function"},{"doc":"Alias for pop_event/2 .","ref":"TimeQueue.html#pop_entry/2","title":"TimeQueue.pop_entry/2","type":"function"},{"doc":"Extracts the next event in the queue with the current system time as now/0 . See pop_event/2 .","ref":"TimeQueue.html#pop_event/1","title":"TimeQueue.pop_event/1","type":"function"},{"doc":"Extracts the next event in the queue according to the given current time in milliseconds. Possible return values are: :empty {:ok, event, new_queue} if the timestamp of the first event is &lt;= to the given current time. The event is deleted from new_queue . {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue . new ( ) |&gt; TimeQueue . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue . pop_event ( tq , _now = 20 ) iex&gt; { :ok , event , _ } = TimeQueue . pop_event ( tq , _now = 100 ) iex&gt; TimeQueue . value ( event ) :hello","ref":"TimeQueue.html#pop_event/2","title":"TimeQueue.pop_event/2","type":"function"},{"doc":"Returns the number of entries in the queue.","ref":"TimeQueue.html#size/1","title":"TimeQueue.size/1","type":"function"},{"doc":"Provides a GenServer compatible timeout from the queue. Accepts the current time as a second argument or will default to the current system time. Returns: :infinity when the queue is empty. 0 when there the next event time has been reached. The delay to the next event otherwise.","ref":"TimeQueue.html#timeout/2","title":"TimeQueue.timeout/2","type":"function"},{"doc":"Returns the time reference of a queue event. This reference is used as a key to identify a unique event. iex&gt; tq = TimeQueue . new ( ) iex&gt; { :ok , tref , tq } = TimeQueue . enqueue ( tq , 10 , :my_value ) iex&gt; Process . sleep ( 10 ) iex&gt; { :ok , event } = TimeQueue . peek_event ( tq ) iex&gt; tref == TimeQueue . tref ( event ) true","ref":"TimeQueue.html#tref/1","title":"TimeQueue.tref/1","type":"function"},{"doc":"Returns the value of a queue event. iex&gt; tq = TimeQueue . new ( ) iex&gt; { :ok , _ , tq } = TimeQueue . enqueue ( tq , 10 , :my_value ) iex&gt; Process . sleep ( 10 ) iex&gt; { :ok , event } = TimeQueue . peek_event ( tq ) iex&gt; TimeQueue . value ( event ) :my_value","ref":"TimeQueue.html#value/1","title":"TimeQueue.value/1","type":"function"},{"doc":"","ref":"TimeQueue.html#t:enqueue_return/0","title":"TimeQueue.enqueue_return/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:event/0","title":"TimeQueue.event/0","type":"opaque"},{"doc":"","ref":"TimeQueue.html#t:event_value/0","title":"TimeQueue.event_value/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:peek_event_return/0","title":"TimeQueue.peek_event_return/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:peek_return/0","title":"TimeQueue.peek_return/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:pop_event_return/0","title":"TimeQueue.pop_event_return/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:pop_return/0","title":"TimeQueue.pop_return/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:t/0","title":"TimeQueue.t/0","type":"opaque"},{"doc":"","ref":"TimeQueue.html#t:timespec/0","title":"TimeQueue.timespec/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:timespec_unit/0","title":"TimeQueue.timespec_unit/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:timestamp_ms/0","title":"TimeQueue.timestamp_ms/0","type":"type"},{"doc":"","ref":"TimeQueue.html#t:tref/0","title":"TimeQueue.tref/0","type":"opaque"},{"doc":"","ref":"TimeQueue.html#t:ttl/0","title":"TimeQueue.ttl/0","type":"type"},{"doc":"Implements a timers queue based on gb_trees . The queue keys are a two-tuple composed of the timestamp of an event and an unique integer. No erlang timers or processes are used, as the queue is only a data structure. The advantage is that the queue can be persisted on storage and keep working after restarting the runtime. The queue maintain its own list of unique integers to avoir relying on BEAM unique integers as they are reset on restart. The main drawback is that the queue entries must be manually checked for expired timers.","ref":"TimeQueue.GbTrees.html","title":"TimeQueue.GbTrees","type":"module"},{"doc":"Deletes an event from the queue and returns the new queue. It accepts a time reference or a full event. When an event is given, its time reference will be used to find the event to delete, meaning the queue event will be deleted even if the value of the passed event was tampered. The function does not fail if the event cannot be found and simply returns the queue as-is.","ref":"TimeQueue.GbTrees.html#delete/2","title":"TimeQueue.GbTrees.delete/2","type":"function"},{"doc":"Deletes all entries from the queue whose values are equal to unwanted . This function is slow with gb_trees , see filter/2 .","ref":"TimeQueue.GbTrees.html#delete_val/2","title":"TimeQueue.GbTrees.delete_val/2","type":"function"},{"doc":"Adds a new event to the queue with a TTL and the current system time as now/0 . See enqueue/4 .","ref":"TimeQueue.GbTrees.html#enqueue/3","title":"TimeQueue.GbTrees.enqueue/3","type":"function"},{"doc":"Adds a new event to the queue with a TTL relative to the given timestamp in milliseconds. Returns {:ok, tref, new_queue} where tref is a timer reference.","ref":"TimeQueue.GbTrees.html#enqueue/4","title":"TimeQueue.GbTrees.enqueue/4","type":"function"},{"doc":"Adds a new event to the queue with an absolute timestamp. Returns {:ok, tref, new_queue} where tref is a timer reference.","ref":"TimeQueue.GbTrees.html#enqueue_abs/3","title":"TimeQueue.GbTrees.enqueue_abs/3","type":"function"},{"doc":"Returns a new queue with entries for whom the given callback returned a truthy value. With the gb trees implementation, this operation is _very expensive as we convert the tree to and ordered list, filter the list, and convert back to a tree.","ref":"TimeQueue.GbTrees.html#filter/2","title":"TimeQueue.GbTrees.filter/2","type":"function"},{"doc":"Returns a new queue with entries for whom the given callback returned a truthy value. Unlinke filter/2 , the callback is only passed the event value. This function is slow with gb_trees , see filter/2 .","ref":"TimeQueue.GbTrees.html#filter_val/2","title":"TimeQueue.GbTrees.filter_val/2","type":"function"},{"doc":"Creates an empty time queue. iex&gt; tq = TimeQueue.GbTrees . new ( ) iex&gt; TimeQueue.GbTrees . peek_event ( tq ) :empty","ref":"TimeQueue.GbTrees.html#new/0","title":"TimeQueue.GbTrees.new/0","type":"function"},{"doc":"Returns the next value of the queue or a delay in milliseconds before the next value. See peek/2 .","ref":"TimeQueue.GbTrees.html#peek/1","title":"TimeQueue.GbTrees.peek/1","type":"function"},{"doc":"Returns the next value of the queue, or a delay, according to the given current time in milliseconds. Just like pop/2 vs. pop_event/2 , peek wil only return {:ok, value} when a timeout is reached whereas peek_event will return {:ok, event} .","ref":"TimeQueue.GbTrees.html#peek/2","title":"TimeQueue.GbTrees.peek/2","type":"function"},{"doc":"Alias for peek_event/1 .","ref":"TimeQueue.GbTrees.html#peek_entry/1","title":"TimeQueue.GbTrees.peek_entry/1","type":"function"},{"doc":"Alias for peek_event/2 .","ref":"TimeQueue.GbTrees.html#peek_entry/2","title":"TimeQueue.GbTrees.peek_entry/2","type":"function"},{"doc":"Returns the next event of the queue or a delay in milliseconds before the next value. event values can be retrieved with TimeQueue.GbTrees.value/1 . See peek_event/2 .","ref":"TimeQueue.GbTrees.html#peek_event/1","title":"TimeQueue.GbTrees.peek_event/1","type":"function"},{"doc":"Returns the next event of the queue according to the given current time in milliseconds. Possible return values are: :empty {:ok, event} if the timestamp of the first event is &lt;= to the given current time. {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue.GbTrees . new ( ) |&gt; TimeQueue.GbTrees . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue.GbTrees . peek_event ( tq , _now = 20 ) iex&gt; { :ok , _ } = TimeQueue.GbTrees . peek_event ( tq , _now = 100 )","ref":"TimeQueue.GbTrees.html#peek_event/2","title":"TimeQueue.GbTrees.peek_event/2","type":"function"},{"doc":"Extracts the next event in the queue or returns a delay. See pop/2 .","ref":"TimeQueue.GbTrees.html#pop/1","title":"TimeQueue.GbTrees.pop/1","type":"function"},{"doc":"Extracts the next event in the queue according to the given current time in milliseconds. Much like pop_event/2 but the tuple returned when an event time is reached (returns with :ok ) success will only contain the value inserted in the queue. Possible return values are: :empty {:ok, value, new_queue} if the timestamp of the first event is &lt;= to the given current time. The event is deleted from new_queue . {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue.GbTrees . new ( ) |&gt; TimeQueue.GbTrees . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue.GbTrees . pop ( tq , _now = 20 ) iex&gt; { :ok , value , _ } = TimeQueue.GbTrees . pop ( tq , _now = 100 ) iex&gt; value :hello","ref":"TimeQueue.GbTrees.html#pop/2","title":"TimeQueue.GbTrees.pop/2","type":"function"},{"doc":"Alias for pop_event/1 .","ref":"TimeQueue.GbTrees.html#pop_entry/1","title":"TimeQueue.GbTrees.pop_entry/1","type":"function"},{"doc":"Alias for pop_event/2 .","ref":"TimeQueue.GbTrees.html#pop_entry/2","title":"TimeQueue.GbTrees.pop_entry/2","type":"function"},{"doc":"Extracts the next event of the queue with the current system time as now/0 . See pop_event/2 .","ref":"TimeQueue.GbTrees.html#pop_event/1","title":"TimeQueue.GbTrees.pop_event/1","type":"function"},{"doc":"Extracts the next event of the queue according to the given current time in milliseconds. Possible return values are: :empty {:ok, event, new_queue} if the timestamp of the first event is &lt;= to the given current time. The event is deleted from new_queue . {:delay, tref, ms} if the timestamp of the first event is &gt; to the given current time. The remaining amount of milliseconds is returned. Example iex&gt; { :ok , tref , tq } = TimeQueue.GbTrees . new ( ) |&gt; TimeQueue.GbTrees . enqueue ( 100 , :hello , _now = 0 ) iex&gt; { :delay , ^ tref , 80 } = TimeQueue.GbTrees . pop_event ( tq , _now = 20 ) iex&gt; { :ok , _ , _ } = TimeQueue.GbTrees . pop_event ( tq , _now = 100 )","ref":"TimeQueue.GbTrees.html#pop_event/2","title":"TimeQueue.GbTrees.pop_event/2","type":"function"},{"doc":"Returns the number of entries in the queue.","ref":"TimeQueue.GbTrees.html#size/1","title":"TimeQueue.GbTrees.size/1","type":"function"},{"doc":"Provides a GenServer compatible timeout from the queue. Accepts the current time as a second argument or will default to the current system time. Returns: :infinity when the queue is empty. 0 when there the next event time has been reached. The delay to the next event otherwise.","ref":"TimeQueue.GbTrees.html#timeout/2","title":"TimeQueue.GbTrees.timeout/2","type":"function"},{"doc":"Returns the time reference of a queue event. This reference is used as a key to identify a unique event. iex&gt; tq = TimeQueue.GbTrees . new ( ) iex&gt; { :ok , tref , tq } = TimeQueue.GbTrees . enqueue ( tq , 10 , :my_value ) iex&gt; Process . sleep ( 10 ) iex&gt; { :ok , event } = TimeQueue.GbTrees . peek_event ( tq ) iex&gt; tref == TimeQueue.GbTrees . tref ( event ) true","ref":"TimeQueue.GbTrees.html#tref/1","title":"TimeQueue.GbTrees.tref/1","type":"function"},{"doc":"Returns the value of a queue event. iex&gt; tq = TimeQueue.GbTrees . new ( ) iex&gt; { :ok , _ , tq } = TimeQueue.GbTrees . enqueue ( tq , 10 , :my_value ) iex&gt; Process . sleep ( 10 ) iex&gt; { :ok , event } = TimeQueue.GbTrees . peek_event ( tq ) iex&gt; TimeQueue.GbTrees . value ( event ) :my_value","ref":"TimeQueue.GbTrees.html#value/1","title":"TimeQueue.GbTrees.value/1","type":"function"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:enqueue_return/0","title":"TimeQueue.GbTrees.enqueue_return/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:event/0","title":"TimeQueue.GbTrees.event/0","type":"opaque"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:event_value/0","title":"TimeQueue.GbTrees.event_value/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:id/0","title":"TimeQueue.GbTrees.id/0","type":"opaque"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:peek_event_return/0","title":"TimeQueue.GbTrees.peek_event_return/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:peek_return/0","title":"TimeQueue.GbTrees.peek_return/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:pop_event_return/0","title":"TimeQueue.GbTrees.pop_event_return/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:pop_return/0","title":"TimeQueue.GbTrees.pop_return/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:t/0","title":"TimeQueue.GbTrees.t/0","type":"opaque"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:timespec/0","title":"TimeQueue.GbTrees.timespec/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:timespec_unit/0","title":"TimeQueue.GbTrees.timespec_unit/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:timestamp_ms/0","title":"TimeQueue.GbTrees.timestamp_ms/0","type":"type"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:tref/0","title":"TimeQueue.GbTrees.tref/0","type":"opaque"},{"doc":"","ref":"TimeQueue.GbTrees.html#t:ttl/0","title":"TimeQueue.GbTrees.ttl/0","type":"type"},{"doc":"This library implements a pure functional queue of timers that is persistable as a simple erlang term. No processes or Erlang timers are used.","ref":"readme.html","title":"TimeQueue","type":"extras"},{"doc":"When you need to attach timers to a persistent data structure (ecto schemas, persistent GenServers, …), for example in a board game. When you need to publish timers over an API (JSON, XML, …).","ref":"readme.html#use-case","title":"TimeQueue - Use case","type":"extras"},{"doc":"Please see the changelog on Github (master branch) .","ref":"readme.html#changelog","title":"TimeQueue - Changelog","type":"extras"},{"doc":"def deps do [ { :time_queue , &quot;~&gt; 0.9&quot; } ] end","ref":"readme.html#installation","title":"TimeQueue - Installation","type":"extras"},{"doc":"tq = TimeQueue . new ( ) { :ok , tref , tq } = TimeQueue . enqueue ( tq , { 500 , :ms } , :myval ) { :delay , ^ tref , _delay } = TimeQueue . peek ( tq ) { :delay , ^ tref , _delay } = TimeQueue . pop ( tq ) Process . sleep ( delay ) # PEEK { :ok , entry } = TimeQueue . peek ( tq ) :myval = TimeQueue . value ( entry ) # POP { :ok , entry , tq } = TimeQueue . pop ( tq ) :myval = TimeQueue . value ( entry ) :empty = TimeQueue . pop ( tq )","ref":"readme.html#basic-usage","title":"TimeQueue - Basic Usage","type":"extras"}]